@@ -3,7 +3,7 @@ use nix::errno::Errno;
 use crate::caps::{errors::CapsError, nr, runtime, Capabilities, Capability};
 
 pub fn clear() -> Result<(), CapsError> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     for flag in Capabilities::all() {
         let cap = flag.try_into().unwrap();
 
@@ -48,7 +48,7 @@ pub fn has_cap(cap: Capability) -> Result<bool, CapsError> {
 pub fn read() -> Result<Capabilities, CapsError> {
     let mut res = Capabilities::empty();
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     for flag in runtime::thread_all_supported() {
         let cap = flag.try_into().unwrap();
 
