@@ -180,7 +180,7 @@ pub(crate) fn sys_openat2(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_open_handler(request, OpenSyscall::Openat2, arg, flags, mode)
 }
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 fn syscall_open_handler(
     request: UNotifyEventRequest,
     syscall: OpenSyscall,
@@ -883,7 +883,7 @@ fn syscall_open_handler(
                     // not want to mess up file offsets for everyone.
                     //
                     // SAFETY: ^^ empty base asserts dir is Some.
-                    #[allow(clippy::disallowed_methods)]
+                    #[expect(clippy::disallowed_methods)]
                     let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
                     let pfd = XPathBuf::from_self_fd(fd);
 
@@ -912,7 +912,7 @@ fn syscall_open_handler(
                         // SAFETY: Substitute machine-id(5) with per-Syd random boot-id.
                         //
                         // Generate machine-id(5) contents.
-                        #[allow(clippy::disallowed_methods)]
+                        #[expect(clippy::disallowed_methods)]
                         let mut machine_id =
                             env::var(ENV_ID).expect("BUG: SYD_ID environment variable isn't set!");
                         machine_id.truncate(32);
@@ -998,7 +998,7 @@ fn handle_crypt(
     mode: Mode,
     opts: CryptOpts,
 ) -> Result<Option<OwnedFd>, Errno> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let files = request.crypt_map.as_ref().unwrap();
     let my_mode = AesMod::from(safe_flags);
 
@@ -1246,7 +1246,7 @@ fn do_open_crypt(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd,
     // Construct path to proc(5) magic symlink.
     //
     // SAFETY: ^^ empty base asserts dir is Some.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
     let pfd = XPathBuf::from_self_fd(fd);
 
