@@ -165,7 +165,7 @@ fn syscall_utime_handler(
     mtime: &TimeSpec,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: SysArg has one element.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
     let fd = path
