@@ -484,7 +484,7 @@ impl XPath {
     /// Enforcing these rules helps prevent security vulnerabilities
     /// arising from unexpected or malicious filenames, such as command
     /// injection, denial of service, or arbitrary file manipulation.
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     pub fn check_name(&self) -> Result<(), Errno> {
         let (_, name) = self.split();
         let name = name.as_bytes();
@@ -554,7 +554,7 @@ impl XPath {
     /// Returns a path that, when joined onto `base`, yields `self`.
     ///
     /// Expects normalized, canonical path.
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     pub fn split_prefix(&self, base: &[u8]) -> Option<&Self> {
         let mut len = base.len();
         if len == 0 {
@@ -590,7 +590,7 @@ impl XPath {
     /// - The function efficiently finds the last `/` in the path and splits at that point.
     /// - Trailing slashes are included in the filename to indicate directory paths.
     /// - For the root path `/`, both parent and filename are the original path reference.
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     pub fn split(&self) -> (&Self, &Self) {
         // Special cases for the empty and root paths.
         let bytes = match self.get(0) {
@@ -641,7 +641,7 @@ impl XPath {
     pub fn extension(&self) -> Option<&Self> {
         let dot = memrchr(b'.', self.as_bytes())?;
         // dot==Some means len>=1.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         if dot < self.0.len() - 1 {
             Some(Self::from_bytes(&self.as_bytes()[dot + 1..]))
         } else {
@@ -655,7 +655,7 @@ impl XPath {
     }
 
     /// Determines the length of the parent path.
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     pub fn parent_len(&self) -> usize {
         // Special cases for the empty and root paths.
         let bytes = match self.get(0) {
@@ -752,8 +752,7 @@ impl XPath {
     /// This function iterates through the bytes of the path from end to
     /// start, and determines whether the last component before any
     /// slashes is a dot.
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::if_same_then_else)]
+    #[expect(clippy::arithmetic_side_effects)]
     pub fn ends_with_dot(&self) -> bool {
         let bytes = self.as_bytes();
 
@@ -893,7 +892,7 @@ impl XPath {
     }
 
     /// Check if path exists.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     pub fn exists(&self, follow: bool) -> bool {
         let flags = if self.is_empty() {
             return false;
@@ -953,7 +952,7 @@ impl XPath {
         let mut start = 0;
 
         // Iterate over '/' positions without allocating.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         for sep in One::new(b'/').iter(bytes) {
             if sep > start {
                 // Non-empty component [start, sep).
@@ -1108,7 +1107,7 @@ impl XPathBuf {
 
         let mut write_pos = 0;
         let mut read_pos = 0;
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         while read_pos < len {
             if self.0[read_pos] == b'/' {
                 // Write a single slash.
@@ -1171,7 +1170,7 @@ impl XPathBuf {
         let new_len = new.len();
         let len = self.0.len();
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match new_len.cmp(&old_len) {
             Ordering::Equal => {
                 // Same length: overwrite the prefix.
@@ -1268,7 +1267,7 @@ impl XPathBuf {
     /// 2. Path must not have a trailing slash!
     #[inline]
     pub unsafe fn pop_unchecked(&mut self) {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         if let Some(idx) = memrchr(b'/', &self.as_bytes()[1..]) {
             self.0.truncate(idx + 1);
         } else if self.0.len() > 1 {
