@@ -442,7 +442,7 @@ macro_rules! skip_unless_kernel_crypto_is_supported {
         };
         match syd::hash::aes_ctr_setup(key_id) {
             Ok(fd) => drop(fd),
-            Err(nix::errno::Errno::EAFNOSUPPORT) => {
+            Err(nix::errno::Errno::EAFNOSUPPORT | Errno::ENOPROTOOPT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
@@ -461,7 +461,7 @@ macro_rules! skip_unless_kernel_crypto_is_supported {
         }
         match syd::hash::hmac_sha256_setup(key_id) {
             Ok(fd) => drop(fd),
-            Err(nix::errno::Errno::EAFNOSUPPORT) => {
+            Err(nix::errno::Errno::EAFNOSUPPORT | Errno::ENOPROTOOPT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
@@ -874,7 +874,10 @@ pub static CI_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("JOB_ID").ok().i
 /// A boolean which specifies if we're running under GitLab CI.
 pub static GL_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("CI_PROJECT_ID").ok().is_some());
 
-pub struct Syd(Command);
+pub struct Syd {
+    cmd: Command,
+    is_quiet: bool,
+}
 
 impl Syd {
     pub fn new(cmd: &str) -> Self {
@@ -882,43 +885,46 @@ impl Syd {
 
         cmd.stdin(Stdio::null());
 
-        Syd(cmd)
+        Syd {
+            cmd,
+            is_quiet: false,
+        }
     }
 
     pub fn c<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-c{}", arg.to_string()));
+        self.cmd.arg(format!("-c{}", arg.to_string()));
         self
     }
 
     pub fn m<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-m{}", arg.to_string()));
+        self.cmd.arg(format!("-m{}", arg.to_string()));
         self
     }
 
     pub fn p<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-p{}", arg.to_string()));
+        self.cmd.arg(format!("-p{}", arg.to_string()));
         self
     }
 
-    #[allow(non_snake_case)]
+    #[expect(non_snake_case)]
     pub fn P<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(format!("-P{}", arg.to_string()));
+        self.cmd.arg(format!("-P{}", arg.to_string()));
         self
     }
 
     pub fn current_dir<S: ToString>(&mut self, arg: S) -> &mut Self {
-        self.0.current_dir(arg.to_string());
+        self.cmd.current_dir(arg.to_string());
         self
     }
 
     pub fn log(&mut self, value: &str) -> &mut Self {
-        self.0
+        self.cmd
             .env(ENV_LOG, env::var(ENV_LOG).unwrap_or(value.to_string()));
         self
     }
 
     pub fn log_fd(&mut self, fd: RawFd) -> &mut Self {
-        self.0.env(ENV_LOG_FD, fd.to_string());
+        self.cmd.env(ENV_LOG_FD, fd.to_string());
         self
     }
 
@@ -928,8 +934,8 @@ impl Syd {
         S: AsRef<OsStr>,
         V: AsRef<OsStr>,
     {
-        self.0.args(["--", &SYD_DO]);
-        self.0.args(args);
+        self.cmd.args(["--", &SYD_DO]);
+        self.cmd.args(args);
         self.do__(value)
     }
 
@@ -937,7 +943,7 @@ impl Syd {
     where
         V: AsRef<OsStr>,
     {
-        self.0.env("SYD_TEST_DO", value);
+        self.cmd.env("SYD_TEST_DO", value);
         self
     }
 
@@ -946,13 +952,13 @@ impl Syd {
         I: IntoIterator<Item = S>,
         S: AsRef<OsStr>,
     {
-        self.0.arg("--");
-        self.0.args(args);
+        self.cmd.arg("--");
+        self.cmd.args(args);
         self
     }
 
     pub fn arg<S: AsRef<OsStr>>(&mut self, arg: S) -> &mut Self {
-        self.0.arg(arg);
+        self.cmd.arg(arg);
         self
     }
 
@@ -961,22 +967,29 @@ impl Syd {
         I: IntoIterator<Item = S>,
         S: AsRef<std::ffi::OsStr>,
     {
-        self.0.args(args);
+        self.cmd.args(args);
         self
     }
 
     pub fn stdin(&mut self, cfg: Stdio) -> &mut Self {
-        self.0.stdin(cfg);
+        self.cmd.stdin(cfg);
         self
     }
 
     pub fn stdout(&mut self, cfg: Stdio) -> &mut Self {
-        self.0.stdout(cfg);
+        self.cmd.stdout(cfg);
         self
     }
 
     pub fn stderr(&mut self, cfg: Stdio) -> &mut Self {
-        self.0.stderr(cfg);
+        self.cmd.stderr(cfg);
+        self
+    }
+
+    pub fn quiet(&mut self) -> &mut Self {
+        self.stderr(Stdio::null());
+        self.stdout(Stdio::null());
+        self.is_quiet = true;
         self
     }
 
@@ -985,7 +998,7 @@ impl Syd {
         K: AsRef<OsStr>,
         V: AsRef<OsStr>,
     {
-        self.0.env(key, value);
+        self.cmd.env(key, value);
         self
     }
 
@@ -993,27 +1006,33 @@ impl Syd {
     where
         K: AsRef<OsStr>,
     {
-        self.0.env_remove(key);
+        self.cmd.env_remove(key);
         self
     }
 
     pub fn spawn(&mut self) -> std::io::Result<Child> {
-        eprintln!("\x1b[93m+ {:?}\x1b[0m", self.0);
-        self.0.spawn()
+        if !self.is_quiet {
+            eprintln!("\x1b[93m+ {:?}\x1b[0m", self.cmd);
+        }
+        self.cmd.spawn()
     }
 
     pub fn status(&mut self) -> std::io::Result<ExitStatus> {
-        eprintln!("\x1b[93m+ {:?}\x1b[0m", self.0);
-        self.0.status()
+        if !self.is_quiet {
+            eprintln!("\x1b[93m+ {:?}\x1b[0m", self.cmd);
+        }
+        self.cmd.status()
     }
 
     pub fn output(&mut self) -> std::io::Result<Output> {
-        eprintln!("\x1b[93m+ {:?}\x1b[0m", self.0);
+        if !self.is_quiet {
+            eprintln!("\x1b[93m+ {:?}\x1b[0m", self.cmd);
+        }
 
         // With `output` stderr defaults to `piped`.
-        self.0.stderr(Stdio::inherit());
+        self.cmd.stderr(Stdio::inherit());
 
-        self.0.output()
+        self.cmd.output()
     }
 }
 
