@@ -61,7 +61,7 @@ pub use crate::unshare::{
 };
 
 /// Main class for running processes. Works in the spirit of builder pattern.
-#[allow(clippy::type_complexity)]
+#[expect(clippy::type_complexity)]
 pub struct Command {
     exe: Executable,
     config: config::Config,
