@@ -15,6 +15,16 @@ use syd::{
     proc::{proc_smaps, proc_statm},
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -55,7 +65,7 @@ syd::main! {
 
     let size = if opt_is_vm {
         match proc_statm(Pid::from_raw(pid)) {
-            Ok(statm) => statm.size.saturating_mul(*syd::config::PAGE_SIZE),
+            Ok(statm) => (statm.size as u64).saturating_mul(*syd::config::PAGE_SIZE),
             Err(error) => {
                 eprintln!("syd-mem: {error}");
                 return Ok(ExitCode::FAILURE);
