@@ -90,6 +90,16 @@ use syd::{
 use tabwriter::TabWriter;
 use tracing_subscriber::layer::SubscriberExt;
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 /// A tracing layer that writes messages to the invalid file descriptor -42.
 pub struct SydLayer;
 
@@ -136,12 +146,12 @@ impl Executor for SydExecutor {
         // 1. Process is None.
         // 2. Process::args is None.
         // 3. Process::args is empty.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let proc = spec
             .process()
             .as_ref()
             .expect("oci_spec::runtime::Spec::process");
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let argv = proc
             .args()
             .as_ref()
@@ -149,7 +159,7 @@ impl Executor for SydExecutor {
             .iter()
             .map(OsString::from)
             .collect::<Vec<_>>();
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let (comm, argv) = argv
             .split_first()
             .map(|(arg0, argv)| (arg0, argv.to_vec()))
@@ -188,7 +198,7 @@ impl Executor for SydExecutor {
         ignore_signals(opts).or(Err(ExecutorError::CantHandle("Failed to ignore signals!")))?;
 
         // Run command under the Syd sandbox and exit with the return code.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         Supervisor::run(sandbox, None, comm, argv, None, None)
             .map(i32::from)
             .map(exit)
@@ -254,7 +264,7 @@ const TENANT_TTY: &str = "tty-";
 // Builder that can be used to configure the properties of a process
 // that will join an existing container sandbox
 struct SydTenantContainerBuilder {
-    #[allow(dead_code)]
+    #[expect(dead_code)]
     base: ContainerBuilder,
     env: HashMap<String, String>,
     cwd: Option<PathBuf>,
@@ -290,14 +300,14 @@ impl SydTenantContainerBuilder {
         if opt.log.is_some() {
             preserve_fds += 1; // Preserve Syd's log file descriptor.
         }
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let builder = ContainerBuilder::new(container_id.clone(), syscall)
             .with_executor(SydExecutor {})
             .with_root_path(opt.root.clone().unwrap())?
             .with_console_socket(args.console_socket.as_ref())
             .with_pid_file(pid_file.clone())?
             .validate_id()?;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         Ok(Self {
             base: builder,
             env: HashMap::new(),
@@ -341,7 +351,7 @@ impl SydTenantContainerBuilder {
         self
     }
 
-    #[allow(dead_code)]
+    #[expect(dead_code)]
     pub fn with_capabilities(mut self, capabilities: Vec<String>) -> Self {
         self.capabilities = capabilities;
         self
@@ -377,7 +387,7 @@ impl SydTenantContainerBuilder {
 
         let notify_path = Self::setup_notify_listener(&container_dir)?;
         // convert path of root file system of the container to absolute path.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let rootfs = fs::canonicalize(spec.root().as_ref().ok_or(MissingSpecError::Root)?.path())
             .map_err(LibcontainerError::OtherIO)?;
 
@@ -391,7 +401,7 @@ impl SydTenantContainerBuilder {
         //FIXME: syd's nix != libcontainer's nix
         //let (read_end, write_end) =
         //    pipe2(OFlag::O_CLOEXEC).map_err(LibcontainerError::OtherSyscall)?;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let (read_end, write_end) = pipe2(OFlag::O_CLOEXEC)
             .map_err(|e| LibcontainerError::OtherIO(std::io::Error::from_raw_os_error(e as i32)))?;
 
@@ -438,7 +448,7 @@ impl SydTenantContainerBuilder {
             let mut buf = [0; 3];
             //FIXME: syd's nix != libcontainer's nix
             //match unistd::read(read_end, &mut buf).map_err(LibcontainerError::OtherSyscall)? {
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             match read(&read_end, &mut buf).map_err(|e| {
                 LibcontainerError::OtherIO(std::io::Error::from_raw_os_error(e as i32))
             })? {
@@ -605,11 +615,11 @@ impl SydTenantContainerBuilder {
         //FIXME: syd's procfs != libcontainer's procfs
         // let init_process = procfs::process::Process::new(container_pid.as_raw())?;
         //let ns = self.get_namespaces(init_process.namespaces()?.0)?;
-        #[allow(clippy::disallowed_types)]
+        #[expect(clippy::disallowed_types)]
         let init_process = procfs::process::Process::new(container_pid.as_raw()).map_err(|_| {
             LibcontainerError::OtherIO(std::io::Error::from_raw_os_error(nix::libc::ESRCH))
         })?;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let ns = self.get_namespaces(
             init_process
                 .namespaces()
@@ -620,7 +630,7 @@ impl SydTenantContainerBuilder {
         )?;
 
         // it should never be the case that linux is not present in spec
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let spec_linux = spec.linux().as_ref().unwrap();
         let mut linux_builder = LinuxBuilder::default().namespaces(ns);
 
@@ -641,10 +651,10 @@ impl SydTenantContainerBuilder {
             ));
         }
 
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let process = utils::open(process).map_err(LibcontainerError::OtherIO)?;
         let reader = BufReader::new(process);
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let process_spec =
             serde_json::from_reader(reader).map_err(LibcontainerError::OtherSerialization)?;
         Ok(process_spec)
@@ -685,7 +695,7 @@ impl SydTenantContainerBuilder {
     ) -> Result<Option<LinuxCapabilities>, LibcontainerError> {
         if !self.capabilities.is_empty() {
             let mut caps: Vec<syd::caps::Capability> = Vec::with_capacity(self.capabilities.len());
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             for cap in &self.capabilities {
                 caps.push(
                     syd::caps::Capability::from_str(cap)
@@ -831,7 +841,7 @@ impl SydTenantContainerBuilder {
 
 // Builder to build a Syd container
 struct SydInitContainerBuilder {
-    #[allow(dead_code)]
+    #[expect(dead_code)]
     base: ContainerBuilder,
     bundle: PathBuf,
     use_systemd: bool,
@@ -866,7 +876,7 @@ impl TryFrom<(GlobalOpts, Create)> for SydInitContainerBuilder {
         if opt.log.is_some() {
             preserve_fds += 1; // Preserve Syd's log file descriptor.
         }
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let builder = ContainerBuilder::new(container_id.clone(), syscall)
             .with_executor(SydExecutor {})
             .with_pid_file(pid_file.clone())?
@@ -874,7 +884,7 @@ impl TryFrom<(GlobalOpts, Create)> for SydInitContainerBuilder {
             .with_root_path(opt.root.clone().unwrap())?
             .with_preserved_fds(preserve_fds)
             .validate_id()?;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         Ok(Self {
             base: builder,
             bundle: args.bundle,
@@ -912,7 +922,7 @@ impl TryFrom<(GlobalOpts, Run)> for SydInitContainerBuilder {
         if opt.log.is_some() {
             preserve_fds += 1; // Preserve Syd's log file descriptor.
         }
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let builder = ContainerBuilder::new(container_id.clone(), syscall)
             .with_executor(SydExecutor {})
             .with_pid_file(pid_file.clone())?
@@ -920,7 +930,7 @@ impl TryFrom<(GlobalOpts, Run)> for SydInitContainerBuilder {
             .with_root_path(opt.root.clone().unwrap())?
             .with_preserved_fds(preserve_fds)
             .validate_id()?;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         Ok(Self {
             base: builder,
             bundle: args.bundle,
@@ -977,7 +987,7 @@ impl SydInitContainerBuilder {
 
         let notify_path = container_dir.join(NOTIFY_FILE);
         // convert path of root file system of the container to absolute path
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let rootfs = fs::canonicalize(spec.root().as_ref().ok_or(MissingSpecError::Root)?.path())
             .map_err(LibcontainerError::OtherIO)?;
 
@@ -1000,7 +1010,7 @@ impl SydInitContainerBuilder {
         let mut config = YoukiConfig::from_spec(&spec, container.id())?;
         let linux = spec.linux().as_ref().ok_or(MissingSpecError::Linux)?;
         config.cgroup_path = get_cgroup_path(linux.cgroups_path(), &self.container_id);
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         config.save(&container_dir).map_err(|err| {
             syd::t!("failed to save config: {err}");
             err
@@ -1044,7 +1054,7 @@ impl SydInitContainerBuilder {
             return Err(LibcontainerError::Exist);
         }
 
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         std::fs::create_dir_all(&container_dir).map_err(|err| {
             syd::t!("failed to create container directory: {err}");
             LibcontainerError::OtherIO(err)
@@ -1068,7 +1078,7 @@ impl SydInitContainerBuilder {
         let syscall = create_syscall();
         let is_rootless_required =
             rootless_required(&*syscall).map_err(LibcontainerError::OtherIO)?;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let syd_dir = if env::var_os(syd::config::ENV_OCI_NO_CONFIG).is_some() {
             None
         } else if !is_rootless_required {
@@ -1102,7 +1112,7 @@ impl SydInitContainerBuilder {
             sources.push(PathBuf::from(format!("{syd_dir}/default.syd-3")));
 
             for path in sources {
-                #[allow(clippy::disallowed_methods)]
+                #[expect(clippy::disallowed_methods)]
                 if XPath::new(&path).exists(true) {
                     // Copy the file into the container root.
                     fs::copy(&path, &target_syd_path)
@@ -1121,7 +1131,7 @@ impl SydInitContainerBuilder {
 
         Self::validate_spec(&spec)?;
 
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         spec.canonicalize_rootfs(&self.bundle).map_err(|err| {
             syd::t!("failed to canonicalize rootfs: {err}");
             err
@@ -1142,7 +1152,7 @@ impl SydInitContainerBuilder {
 
         if let Some(process) = spec.process() {
             if let Some(_profile) = process.apparmor_profile() {
-                #[allow(clippy::disallowed_methods)]
+                #[expect(clippy::disallowed_methods)]
                 let apparmor_is_enabled = apparmor::is_enabled().map_err(|err| {
                     syd::t!("failed to check if apparmor is enabled");
                     LibcontainerError::OtherIO(err)
@@ -1301,7 +1311,7 @@ impl SydContainerBuilderImpl {
         // is not writeable unless you're an privileged user (if !dumpable is
         // set). All children inherit their parent's oom_score_adj value on
         // fork(2) so this will always be propagated properly.
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         if let Some(oom_score_adj) = process.oom_score_adj() {
             syd::t!("Set OOM score to {oom_score_adj}");
             let mut f = fs::File::create("/proc/self/oom_score_adj").map_err(|err| {
@@ -1363,7 +1373,7 @@ impl SydContainerBuilderImpl {
             as_sibling: self.as_sibling,
         };
 
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let (init_pid, need_to_clean_up_intel_rdt_dir) =
             process::container_main_process::container_main_process(&container_args).map_err(
                 |err| {
@@ -1373,7 +1383,7 @@ impl SydContainerBuilderImpl {
             )?;
 
         // if file to write the pid to is specified, write pid of the child
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         if let Some(pid_file) = &self.pid_file {
             fs::write(pid_file, format!("{init_pid}")).map_err(|err| {
                 syd::t!("failed to write pid to file: {err}");
@@ -1564,7 +1574,7 @@ fn cmd_create(opt: GlobalOpts, args: Create) -> SydResult<ExitCode> {
 }
 
 fn cmd_start(opt: GlobalOpts, args: Start) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1577,7 +1587,7 @@ fn cmd_start(opt: GlobalOpts, args: Start) -> SydResult<ExitCode> {
 }
 
 fn cmd_state(opt: GlobalOpts, args: State) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1590,7 +1600,7 @@ fn cmd_state(opt: GlobalOpts, args: State) -> SydResult<ExitCode> {
 }
 
 fn cmd_kill(opt: GlobalOpts, args: Kill) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1605,7 +1615,7 @@ fn cmd_kill(opt: GlobalOpts, args: Kill) -> SydResult<ExitCode> {
 }
 
 fn cmd_delete(opt: GlobalOpts, args: Delete) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(false) && args.force {
         return Ok(ExitCode::SUCCESS);
@@ -1617,7 +1627,7 @@ fn cmd_delete(opt: GlobalOpts, args: Delete) -> SydResult<ExitCode> {
     Ok(ExitCode::SUCCESS)
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn cmd_features(_opt: GlobalOpts, _args: Features) -> SydResult<ExitCode> {
     // Query supported namespaces
     let namespaces = match query_supported_namespaces() {
@@ -1683,7 +1693,7 @@ fn cmd_features(_opt: GlobalOpts, _args: Features) -> SydResult<ExitCode> {
 }
 
 fn cmd_ps(opt: GlobalOpts, args: Ps) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1739,7 +1749,7 @@ fn cmd_list(opt: GlobalOpts, _args: List) -> SydResult<ExitCode> {
     let mut content = String::new();
     // all containers' data is stored in their respective dir in root directory
     // so we iterate through each and print the various info
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     for container_dir in fs::read_dir(opt.root.unwrap())? {
         let container_dir = container_dir?.path();
         let state_file = container_dir.join("state.json");
@@ -1790,7 +1800,7 @@ fn cmd_spec(_opt: GlobalOpts, args: liboci_cli::Spec) -> SydResult<ExitCode> {
     };
 
     // write data to config.json
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let file = fs::File::create("config.json")?;
     let mut writer = BufWriter::new(file);
     to_writer_pretty(&mut writer, &spec)?;
@@ -1805,7 +1815,7 @@ fn cmd_spec(_opt: GlobalOpts, args: liboci_cli::Spec) -> SydResult<ExitCode> {
 // https://man7.org/linux/man-pages/man7/cgroups.7.html
 // https://www.kernel.org/doc/Documentation/cgroup-v1/freezer-subsystem.txt
 fn cmd_pause(opt: GlobalOpts, args: Pause) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1823,7 +1833,7 @@ fn cmd_pause(opt: GlobalOpts, args: Pause) -> SydResult<ExitCode> {
 // https://man7.org/linux/man-pages/man7/cgroups.7.html
 // https://www.kernel.org/doc/Documentation/cgroup-v1/freezer-subsystem.txt
 fn cmd_resume(opt: GlobalOpts, args: Resume) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1836,7 +1846,7 @@ fn cmd_resume(opt: GlobalOpts, args: Resume) -> SydResult<ExitCode> {
 }
 
 fn cmd_events(opt: GlobalOpts, args: Events) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1849,7 +1859,7 @@ fn cmd_events(opt: GlobalOpts, args: Events) -> SydResult<ExitCode> {
 }
 
 fn cmd_update(opt: GlobalOpts, args: Update) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1863,7 +1873,7 @@ fn cmd_update(opt: GlobalOpts, args: Update) -> SydResult<ExitCode> {
     })?;
 
     let linux_res: LinuxResources;
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     if let Some(resources_path) = args.resources {
         linux_res = if resources_path.to_string_lossy() == "-" {
             serde_json::from_reader(std::io::stdin())?
@@ -1891,7 +1901,7 @@ fn cmd_update(opt: GlobalOpts, args: Update) -> SydResult<ExitCode> {
 }
 
 fn cmd_checkpoint(opt: GlobalOpts, args: Checkpoint) -> SydResult<ExitCode> {
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let container_root = opt.root.unwrap().join(args.container_id.clone());
     if !XPath::new(&container_root).exists(true) {
         return Err(Errno::ENOENT.into());
@@ -1974,7 +1984,7 @@ fn cmd_run(opt: GlobalOpts, args: Run) -> SydResult<ExitCode> {
     );
     //FIXME: syd's nix != libcontainer's nix
     //let foreground_result = handle_foreground(container.pid().unwrap());
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let foreground_result = handle_foreground(Pid::from_raw(container.pid().unwrap().as_raw()));
     // execute the destruction action after the container finishes running
     container.delete(true)?;
@@ -2186,7 +2196,7 @@ fn handle_foreground(init_pid: Pid) -> SydResult<i32> {
             signal => {
                 syd::t!("forwarding signal {}", signal as i32);
                 // There is nothing we can do if we fail to forward the signal.
-                #[allow(clippy::disallowed_methods)]
+                #[expect(clippy::disallowed_methods)]
                 let _ = kill(init_pid, Some(signal)).map_err(|_err| {
                     syd::t!("failed to forward signal to container init process: {_err}")
                 });
@@ -2277,7 +2287,7 @@ fn make_root(opt: &mut GlobalOpts) -> SydResult<()> {
     let uid = Uid::current();
 
     // Determine root path.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     if opt.root.is_none() {
         let syscall = create_syscall();
         let is_rootless_required = rootless_required(&*syscall)?;
@@ -2347,7 +2357,7 @@ fn is_executable(path: &Path) -> std::result::Result<bool, std::io::Error> {
     Ok(metadata.is_file() && permissions.mode() & 0o001 != 0)
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn set_name(name: &str) -> std::result::Result<(), Errno> {
     let name = CString::new(name).map_err(|_| Errno::EINVAL)?;
 
