@@ -29,14 +29,12 @@ use nix::{
 };
 use syd::{
     compat::epoll_ctl_safe,
+    config::VDSO_SYSCALLS,
     confine::{confine_mdwe, extend_ioctl},
     err::SydResult,
     fs::{closeexcept, set_exclusive, set_nonblock, winsize_get, winsize_set},
     ignore_signals,
-    ioctl::{
-        TCGETS, TCGETS2, TCSETS, TCSETS2, TCSETSF, TCSETSF2, TCSETSW, TCSETSW2, TIOCGWINSZ,
-        TIOCSWINSZ,
-    },
+    ioctl::{TCGETS, TCGETS2, TCSETS, TCSETS2, TIOCGWINSZ, TIOCSWINSZ},
     landlock_policy::LandlockPolicy,
     rng::duprand,
     IgnoreSignalOpts,
@@ -62,7 +60,7 @@ syd::main! {
     // 2. The PID fd and the PTY fd passed by the Syd process.
     // Nothing can be done on closeexcept errors.
     // We do it early here so FD randomization doesn't effect performance.
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_sign_loss)]
     {
         let fd1 = opts.fpid.as_raw_fd() as libc::c_uint;
         let fd2 = opts.fpty.as_raw_fd() as libc::c_uint;
@@ -206,7 +204,7 @@ syd::main! {
     );
 
     if let Some(tio) = tio {
-        let _ = tcsetattr(&fstd_rd, SetArg::TCSAFLUSH, &tio);
+        let _ = tcsetattr(&fstd_rd, SetArg::TCSANOW, &tio);
     }
 
     Ok(match result {
@@ -216,7 +214,7 @@ syd::main! {
 }
 
 /// Run the PTY forwarder.
-#[allow(clippy::too_many_arguments)]
+#[expect(clippy::too_many_arguments)]
 fn run_pty_forwarder<
     F1: AsFd,
     F2: AsFd,
@@ -394,11 +392,12 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
         "exit_group",
         // can handle signals limitedly.
         "sigaltstack",
-        // can handle memory limitedly.
+        // can {{dr}e,}allocate memory.
+        // mmap{,2} and mprotect are further confined to disable PROT_EXEC.
         "brk",
+        "madvise",
+        "mremap",
         "munmap",
-        // can generate random bytes.
-        "getrandom",
         // can close files.
         "close",
         // can do I/O with splice.
@@ -412,14 +411,14 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
     ];
 
     // Default allowlist.
-    for name in allow_call {
+    for name in allow_call.iter().chain(VDSO_SYSCALLS) {
         if let Ok(syscall) = ScmpSyscall::from_name(name) {
             ctx.add_rule(ScmpAction::Allow, syscall)?;
         }
     }
 
     // Allow read(2) to the signal fd only.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let syscall = ScmpSyscall::from_name("read").unwrap();
     ctx.add_rule_conditional(
         ScmpAction::Allow,
@@ -430,26 +429,20 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
     // Allow ioctl(2) requests:
     // 1. TCGETS{,2}, aka tcgetattr(3)
     // 2. TCSETS{,2}, aka tcsetattr(3) with TCSANOW
-    // 3. TCSETSW{,2}, aka tcsetattr(3) with TCSADRAIN
-    // 4. TCSETSF{,2}, aka tcsetattr(3) with TCSAFLUSH
-    // 5. TIOCGWINSZ, aka winsize_get
-    // 6. TIOCSWINSZ, aka winsize_set
+    // 3. TIOCGWINSZ, aka winsize_get
+    // 4. TIOCSWINSZ, aka winsize_set
     //
     // For *2, we use hardcoded versions because not all libcs define them.
     // They are portable as `struct termios2` has the same size
     // across 32-bit and 64-bit architectures.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let syscall = ScmpSyscall::from_name("ioctl").unwrap();
-    #[allow(clippy::unnecessary_cast)]
+    #[expect(clippy::unnecessary_cast)]
     for request in [
         *TCGETS,
         *TCGETS2,
         *TCSETS,
         *TCSETS2,
-        *TCSETSW,
-        *TCSETSW2,
-        *TCSETSF,
-        *TCSETSF2,
         *TIOCGWINSZ,
         *TIOCSWINSZ,
     ] {
@@ -477,6 +470,26 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
         }
     }
 
+    // Allow safe prctl(2) operations.
+    let sysname = "prctl";
+    if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
+        for op in [libc::PR_SET_VMA as u64] {
+            ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == op)])?;
+        }
+    }
+
+    // Prevent executable memory.
+    const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
+    for name in ["mmap", "mmap2", "mprotect"] {
+        #[expect(clippy::disallowed_methods)]
+        let syscall = ScmpSyscall::from_name(name).unwrap();
+        ctx.add_rule_conditional(
+            ScmpAction::Allow,
+            syscall,
+            &[scmp_cmp!($arg2 & PROT_EXEC == 0)],
+        )?;
+    }
+
     // We will ignore unshare errors next step and here we keep
     // with the expectation that we're inside the safe directory.
     chdir("/proc/self/fdinfo")?;
@@ -518,11 +531,6 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
 
         // Set the process dumpable attribute to not-dumpable.
         let _ = set_dumpable(false);
-
-        // Deny reading the timestamp counter (x86 only).
-        // SAFETY: Our nix version does not have a wrapper for SET_TSC yet.
-        #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
-        let _ = unsafe { libc::prctl(libc::PR_SET_TSC, libc::PR_TSC_SIGSEGV) };
     }
 
     if print_rules {
@@ -636,14 +644,14 @@ fn refresh_pty<Fd1: AsFd, Fd2: AsFd>(src: Fd1, dst: Fd2) -> Option<Termios> {
     let orig_tio = tio.clone();
 
     // Inherit host terminal settings for PTY.
-    let _ = tcsetattr(&dst, SetArg::TCSAFLUSH, &tio);
+    let _ = tcsetattr(&dst, SetArg::TCSANOW, &tio);
 
     // Set raw mode for input TTY.
     // Enable output processing.
     cfmakeraw(&mut tio);
     tio.output_flags.insert(OutputFlags::OPOST);
 
-    let _ = tcsetattr(&src, SetArg::TCSAFLUSH, &tio);
+    let _ = tcsetattr(&src, SetArg::TCSANOW, &tio);
 
     Some(orig_tio)
 }
