@@ -12,6 +12,16 @@ use nix::{libc::pid_t, unistd::Pid};
 use serde_json::json;
 use syd::proc::{proc_stat, proc_status};
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     syd::set_sigpipe_dfl()?;
 
@@ -29,7 +39,7 @@ syd::main! {
 
     let stat = proc_stat(Pid::from_raw(pid))?;
     let status = proc_status(Pid::from_raw(pid))?;
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let status = json!({
         "pid": pid,
         "tgid": status.pid,
@@ -47,7 +57,7 @@ syd::main! {
         },
     });
 
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let status = serde_json::to_string(&status).expect("JSON");
     println!("{status}");
 
