@@ -27,6 +27,16 @@ use syd::{
     wildmatch::inamematch,
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -34,6 +44,7 @@ syd::main! {
 
     // Parse CLI options.
     let mut opt_arch = ScmpArch::native(); // -a
+    let mut opt_errno = false; // -e
     let mut opt_ioctl = false; // -i
     let mut opt_ghost = false; // -g
     let mut opt_probe = false; // -p
@@ -50,6 +61,7 @@ syd::main! {
                 help();
                 return Ok(ExitCode::SUCCESS);
             }
+            Short('e') => opt_errno = true,
             Short('i') => opt_ioctl = true,
             Short('p') => opt_probe = true,
             Short('o') => opt_open = true,
@@ -91,16 +103,18 @@ syd::main! {
         }
     }
 
-    let flags = [opt_ioctl, opt_open, opt_signal];
+    let flags = [opt_errno, opt_ioctl, opt_open, opt_signal];
     if flags.iter().filter(|&&flag| flag).count() > 1 {
-        eprintln!("At most one of -i, -o and -s must be given!");
+        eprintln!("At most one of -e, -i, -o and -s must be given!");
         return Err(Errno::EINVAL.into());
     }
 
     let sysarg = if let Some(value) = opt_sys {
         value
     } else {
-        let what = if opt_ioctl {
+        let what = if opt_errno {
+            "errno"
+        } else if opt_ioctl {
             "ioctl"
         } else if opt_open {
             "open"
@@ -113,16 +127,58 @@ syd::main! {
         return Ok(ExitCode::FAILURE);
     };
 
-    if opt_ioctl {
-        match sysarg.parse::<Ioctl>() {
+    if opt_errno {
+        return match sysarg.parse::<u16>() {
+            Ok(0) => Ok(ExitCode::FAILURE),
+            Ok(num) => {
+                // number -> name
+                let errno = Errno::from_raw(i32::from(num));
+                if errno == Errno::UnknownErrno {
+                    return Ok(ExitCode::FAILURE);
+                }
+                let estr = errno.to_string();
+                let mut iter = estr.split(": ");
+                let name = iter.next().unwrap_or("?");
+                let desc = iter.next().unwrap_or("?");
+                println!("{num}\t{name}\t{desc}");
+                Ok(ExitCode::SUCCESS)
+            }
+            Err(_) => {
+                // glob -> [number]
+                let glob = sysarg.to_str().ok_or(Errno::EINVAL)?;
+
+                let mut ok = false;
+                for errno in (1..=4096).map(|n| Errno::from_raw(i32::from(n))) {
+                    if errno == Errno::UnknownErrno {
+                        continue;
+                    }
+                    let estr = errno.to_string();
+                    let mut iter = estr.split(": ");
+                    let name = iter.next().unwrap_or("?");
+                    let desc = iter.next().unwrap_or("?");
+                    if inamematch(glob, &estr) {
+                        println!("{}\t{}\t{}", errno as i32, name, desc);
+                        ok = true;
+                    }
+                }
+
+                Ok(if ok {
+                    ExitCode::SUCCESS
+                } else {
+                    ExitCode::FAILURE
+                })
+            }
+        };
+    } else if opt_ioctl {
+        return match sysarg.parse::<Ioctl>() {
             Ok(num) => {
                 if let Some(names) = ioctl_names_get(num, opt_arch) {
                     for name in names {
                         println!("{name}\t{num}");
                     }
-                    return Ok(ExitCode::SUCCESS);
+                    Ok(ExitCode::SUCCESS)
                 } else {
-                    return Ok(ExitCode::FAILURE);
+                    Ok(ExitCode::FAILURE)
                 }
             }
             Err(_) => {
@@ -139,13 +195,13 @@ syd::main! {
                     }
                 }
 
-                return Ok(if ok {
+                Ok(if ok {
                     ExitCode::SUCCESS
                 } else {
                     ExitCode::FAILURE
-                });
+                })
             }
-        }
+        };
     } else if opt_open {
         return match sysarg
             .parse::<c_int>()
@@ -293,7 +349,6 @@ syd::main! {
 
     // Prepare system call arguments.
     let mut args: [Option<libc::c_long>; 6] = [None; 6];
-    #[allow(clippy::needless_range_loop)]
     for argc in 0..6 {
         if let Some(value) = opt_arg.get(argc) {
             args[argc] = match value.parse::<libc::c_long>() {
@@ -316,15 +371,16 @@ syd::main! {
 }
 
 fn help() {
-    println!("Usage: syd-sys [-hgiopst] [-a list|native|x86|x86_64|aarch64...] number|name-glob [<probe-args>...]");
+    println!("Usage: syd-sys [-hgeiopst] [-a list|native|x86|x86_64|aarch64...] number|name-glob [<probe-args>...]");
     println!("Given a number, print the matching syscall name and exit.");
     println!("Given a glob, print case-insensitively matching syscall names and exit.");
+    println!("Given -e, query errnos rather than syscalls.");
     println!("Given -i, query ioctls rather than syscalls.");
     println!("Given -o, query open flags rather than syscalls.");
     println!("Given -s, query signals rather than syscalls.");
     println!("Given -p, probe the system call and print result.");
     println!("Given -g with -p, enable Syd's Ghost mode prior to probing.");
-    println!("Specify syscall probe timeout in seconds, defaults to 3 seconds.");
+    println!("Use -t to specify syscall probe timeout in seconds, defaults to 3 seconds.");
 }
 
 fn probe_syscall(
@@ -340,7 +396,7 @@ fn probe_syscall(
         .rev()
         .find(|&(_, elem)| elem.is_some())
         .map_or(0, |(idx, _)| idx + 1);
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     match unsafe { fork() }.expect("fork") {
         ForkResult::Child => unsafe {
             match argc {
