@@ -20,7 +20,7 @@ use std::{
     thread,
 };
 
-#[allow(deprecated)]
+#[expect(deprecated)]
 use libc::SOCK_PACKET;
 use libc::{AF_ALG, AF_INET, AF_INET6, AF_NETLINK, AF_UNIX, EACCES, SOCK_RAW};
 use libseccomp::{scmp_cmp, RawSyscall, ScmpAction, ScmpArch, ScmpFilterContext, ScmpSyscall};
@@ -49,7 +49,8 @@ use crate::{
         FTRUNCATE64_COOKIE_ARG4, FTRUNCATE64_COOKIE_ARG5, FTRUNCATE_COOKIE_ARG2,
         FTRUNCATE_COOKIE_ARG3, FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5, LINKAT_COOKIE_ARG5,
         MEMFD_CREATE_COOKIE_ARG2, MEMFD_CREATE_COOKIE_ARG3, MEMFD_CREATE_COOKIE_ARG4,
-        MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5,
+        MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5, PIPE2_COOKIE_ARG2,
+        PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4, PIPE2_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5,
         SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4,
         SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3,
         SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5,
@@ -88,7 +89,7 @@ pub(crate) struct Worker {
 }
 
 impl Worker {
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub(crate) fn new(
         fd: RawFd,
         cache: Arc<WorkerCache<'static>>,
@@ -118,7 +119,7 @@ impl Worker {
         }
     }
 
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn try_spawn(
         self,
         ctx: Option<&ScmpFilterContext>,
@@ -265,7 +266,7 @@ impl Worker {
         if let Some(timer) = timer {
             // SAFETY: Something is awfully wrong if we cannot
             // set the timer so we panic here to indicate this.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             timer.start().expect("timer_settime");
         }
 
@@ -276,7 +277,7 @@ impl Worker {
 
         if let Some(timer) = timer {
             // SAFETY: See above.
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             timer.stop().expect("timer_settime");
         }
         res?;
@@ -293,7 +294,7 @@ impl Worker {
         }
     }
 
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     fn handle(&self, mut req: ScmpNotifReq) {
         // Correct architecture for x32 syscalls.
         if req.data.arch == ScmpArch::X8664
@@ -373,7 +374,7 @@ impl Worker {
     }
 
     /// Confine Worker thread.
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn prepare_confine(
         seccomp_fd: RawFd,
         flags: Flags,
@@ -433,10 +434,10 @@ impl Worker {
         // Syscall argument cookies for SECCOMP_IOCTL_NOTIF_SEND my be disabled
         // at startup with trace/allow_unsafe_nocookie:1.
         let sysname = "ioctl";
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         match ScmpSyscall::from_name(sysname) {
             Ok(syscall) => {
-                #[allow(clippy::useless_conversion)]
+                #[expect(clippy::useless_conversion)]
                 for ioctl_request in SECCOMP_IOCTL_NOTIF_LIST {
                     let mut rules = vec![scmp_cmp!($arg0 == seccomp_fd as u64)];
 
@@ -502,22 +503,28 @@ impl Worker {
             }
         }
 
-        // Deny pipe2(2) O_NOTIFICATION_PIPE flag.
+        // Restrict pipe2(2) flags.
+        // Restrict pipe2(2) using syscall argument cookies.
         let sysname = "pipe2";
-        #[allow(clippy::cast_sign_loss)]
-        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
-            // O_NOTIFICATION_PIPE is equivalent to O_EXCL,
-            // see: linux/watch_queue.h
-            const O_NOTIFICATION_PIPE: u64 = OFlag::O_EXCL.bits() as u64;
+        #[expect(clippy::useless_conversion)]
+        match ScmpSyscall::from_name(sysname) {
+            Ok(syscall) => {
+                let mut rules = vec![scmp_cmp!($arg1 == OFlag::O_CLOEXEC.bits() as u64)];
+                if !flags.allow_unsafe_nocookie() {
+                    rules.extend(&[
+                        scmp_cmp!($arg2 == (*PIPE2_COOKIE_ARG2).into()),
+                        scmp_cmp!($arg3 == (*PIPE2_COOKIE_ARG3).into()),
+                        scmp_cmp!($arg4 == (*PIPE2_COOKIE_ARG4).into()),
+                        scmp_cmp!($arg5 == (*PIPE2_COOKIE_ARG5).into()),
+                    ]);
+                }
 
-            ctx.add_rule_conditional(
-                ScmpAction::Allow,
-                syscall,
-                &[scmp_cmp!($arg1 & O_NOTIFICATION_PIPE == 0)],
-            )?;
-        } else {
-            info!("ctx": "confine", "op": "allow_emu_syscall",
-                "msg": format!("invalid or unsupported syscall {sysname}"));
+                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
+            }
+            Err(_) => {
+                info!("ctx": "confine", "op": "allow_emu_syscall",
+                    "msg": format!("invalid or unsupported syscall {sysname}"));
+            }
         }
 
         // Allow safe fcntl(2) utility calls.
@@ -536,6 +543,24 @@ impl Worker {
             }
         }
 
+        // Allow safe prctl(2) operations.
+        let sysname = "prctl";
+        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
+            let safe_setid =
+                flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+            let iter = EMU_PRCTL_OPS.iter().chain(if safe_setid {
+                EMU_PRCTL_OPS_SAFESETID.iter()
+            } else {
+                [].iter()
+            });
+            for (_, op) in iter {
+                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == *op)])?;
+            }
+        } else {
+            info!("ctx": "confine", "op": "allow_emu_syscall",
+                "msg": format!("invalid or unsupported syscall {sysname}"));
+        }
+
         // Deny installing new signal handlers for {rt_,}sigaction(2).
         for sysname in ["sigaction", "rt_sigaction"] {
             let syscall = match ScmpSyscall::from_name(sysname) {
@@ -553,7 +578,7 @@ impl Worker {
         }
 
         // Allow safe system calls.
-        for sysname in EMU_SYSCALLS {
+        for sysname in EMU_SYSCALLS.iter().chain(VDSO_SYSCALLS) {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
@@ -570,7 +595,7 @@ impl Worker {
         if !flags.allow_unsafe_nocookie() {
             // memfd_create(2) may be used only with syscall argument cookies.
             let sysname = "memfd_create";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -593,7 +618,7 @@ impl Worker {
             // renameat2(2) may be used only with syscall argument cookies.
             // We also prevent AT_FDCWD usage as fd argument.
             let sysname = "renameat2";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -613,7 +638,7 @@ impl Worker {
 
             // {,f}truncate{,64}(2) may be used only with syscall argument cookies.
             let sysname = "truncate";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -634,7 +659,7 @@ impl Worker {
             }
 
             let sysname = "truncate64";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -655,7 +680,7 @@ impl Worker {
             }
 
             let sysname = "ftruncate";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -676,7 +701,7 @@ impl Worker {
             }
 
             let sysname = "ftruncate64";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -700,8 +725,8 @@ impl Worker {
             // We also enforce PROC_FD usage as fd argument.
             // This fd is randomized so it further raises the bar.
             let sysname = "linkat";
-            #[allow(clippy::cast_sign_loss)]
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::cast_sign_loss)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -722,7 +747,7 @@ impl Worker {
             // unlinkat(2) may be used only with syscall argument cookies.
             // We also prevent AT_FDCWD usage as fd argument.
             let sysname = "unlinkat";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -745,7 +770,7 @@ impl Worker {
             // openat2(2) may be used only with syscall argument cookies.
             // We also prevent AT_FDCWD usage as fd argument.
             let sysname = "openat2";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -766,7 +791,7 @@ impl Worker {
 
             // uname(2) may be used only with syscall argument cookies.
             let sysname = "uname";
-            #[allow(clippy::useless_conversion)]
+            #[expect(clippy::useless_conversion)]
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule_conditional(
@@ -815,7 +840,6 @@ impl Worker {
         // openat(2) may be used to open the parent directory only by getdir_long().
         // The rest of the attempts are denied with ENOSYS for compat.
         let sysname = "openat";
-        #[allow(clippy::cast_sign_loss)]
         match ScmpSyscall::from_name(sysname) {
             Ok(syscall) => {
                 let dotdot = dotdot_with_nul();
@@ -855,10 +879,10 @@ impl Worker {
         //
         // We only enforce this on architectures where the system call is direct,
         // and there's no socketcall(2) multiplexer indirection.
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::useless_conversion)]
-        #[allow(deprecated)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
+        #[expect(clippy::useless_conversion)]
+        #[expect(deprecated)]
         if let Some(syscall) = SYS_SOCKET.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall)) {
             // Deny based on socket type.
             if !flags.allow_unsafe_socket() {
@@ -920,10 +944,8 @@ impl Worker {
         //
         // We only enforce this on architectures where the system call is direct,
         // and there's no socketcall(2) multiplexer indirection.
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::useless_conversion)]
-        #[allow(deprecated)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::useless_conversion)]
         if let Some(syscall) = SYS_ACCEPT4.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall)) {
             // Secure using syscall argument cookies.
             let mut rules = vec![];
@@ -956,10 +978,8 @@ impl Worker {
         //
         // We only enforce this on architectures where the system call is direct,
         // and there's no socketcall(2) multiplexer indirection.
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::useless_conversion)]
-        #[allow(deprecated)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::useless_conversion)]
         if let Some(syscall) = SYS_BIND.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall)) {
             // Secure using syscall argument cookies.
             let mut rules = vec![];
@@ -993,10 +1013,8 @@ impl Worker {
         //
         // We only enforce this on architectures where the system call is direct,
         // and there's no socketcall(2) multiplexer indirection.
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::useless_conversion)]
-        #[allow(deprecated)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::useless_conversion)]
         if let Some(syscall) = SYS_CONNECT.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall)) {
             // Secure using syscall argument cookies.
             let mut rules = vec![];
@@ -1055,7 +1073,7 @@ impl Worker {
         // Allow execveat(2) with AT_EXECVE_CHECK for Linux>=6.14.
         if *HAVE_AT_EXECVE_CHECK {
             let sysname = "execveat";
-            #[allow(clippy::cast_sign_loss)]
+            #[expect(clippy::cast_sign_loss)]
             let atcheck = AT_EXECVE_CHECK.bits() as u64;
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
@@ -1129,7 +1147,7 @@ impl<'a> Sentinel<'a> {
         }
     }
 
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     fn deny_syscall(&self, seccomp_id: u64, errno: Errno) {
         let response = libc::seccomp_notif_resp {
             id: seccomp_id,
