@@ -22,14 +22,21 @@ const SYD_SIZE_EXE: &'static str = env!("CARGO_BIN_EXE_syd-size");
 const SYD_AES_EXE: &'static str = env!("CARGO_BIN_EXE_syd-aes");
 const SYD_KEY_EXE: &'static str = env!("CARGO_BIN_EXE_syd-key");
 const SYD_LOCK_EXE: &'static str = env!("CARGO_BIN_EXE_syd-lock");
+const SYD_MDWE_EXE: &'static str = env!("CARGO_BIN_EXE_syd-mdwe");
+const SYD_OFD_EXE: &'static str = env!("CARGO_BIN_EXE_syd-ofd");
+const SYD_PAUSE_EXE: &'static str = env!("CARGO_BIN_EXE_syd-pause");
 const SYD_PDS_EXE: &'static str = env!("CARGO_BIN_EXE_syd-pds");
 const SYD_ELF_EXE: &'static str = env!("CARGO_BIN_EXE_syd-elf");
 const SYD_CPU_EXE: &'static str = env!("CARGO_BIN_EXE_syd-cpu");
 const SYD_TOR_EXE: &'static str = env!("CARGO_BIN_EXE_syd-tor");
 const SYD_TEST_EXE: &'static str = env!("CARGO_BIN_EXE_syd-test");
 const SYD_TEST_DO_EXE: &'static str = env!("CARGO_BIN_EXE_syd-test-do");
+const SYD_UTS: &'static str = env!("CARGO_BIN_EXE_syd-uts");
 const SYD_X: &'static str = env!("CARGO_BIN_EXE_syd-x");
 
+#[cfg(feature = "oci")]
+const SYD_OCI: &'static str = env!("CARGO_BIN_EXE_syd-oci");
+
 #[test]
 fn syd_test() {
     env::set_var("CARGO_BIN_EXE_syd", SYD_EXE);
@@ -45,12 +52,20 @@ fn syd_test() {
     env::set_var("CARGO_BIN_EXE_syd-aes", SYD_AES_EXE);
     env::set_var("CARGO_BIN_EXE_syd-key", SYD_KEY_EXE);
     env::set_var("CARGO_BIN_EXE_syd-lock", SYD_LOCK_EXE);
+    env::set_var("CARGO_BIN_EXE_syd-mdwe", SYD_MDWE_EXE);
+    env::set_var("CARGO_BIN_EXE_syd-ofd", SYD_OFD_EXE);
+    env::set_var("CARGO_BIN_EXE_syd-pause", SYD_PAUSE_EXE);
     env::set_var("CARGO_BIN_EXE_syd-pds", SYD_PDS_EXE);
     env::set_var("CARGO_BIN_EXE_syd-elf", SYD_ELF_EXE);
     env::set_var("CARGO_BIN_EXE_syd-cpu", SYD_CPU_EXE);
     env::set_var("CARGO_BIN_EXE_syd-tor", SYD_TOR_EXE);
     env::set_var("CARGO_BIN_EXE_syd-test-do", SYD_TEST_DO_EXE);
+    env::set_var("CARGO_BIN_EXE_syd-uts", SYD_UTS);
     env::set_var("CARGO_BIN_EXE_syd-x", SYD_X);
+
+    #[cfg(feature = "oci")]
+    env::set_var("CARGO_BIN_EXE_syd-oci", SYD_OCI);
+
     let status = Command::new(SYD_TEST_EXE)
         .status()
         .expect("execute syd-test");
